app_name = "EU Diktátor 2020"

what_is_your_name = "Jak se jmenuješ? "
what_is_your_gender = "Jakého seš pohlaví? "

proceed = "Pokračovat..."

period = "Období: {} {}"
budget = "Rozpočet {}: {:,} EUR, rozdíl příjmů a výdajů: {:,} EUR, dluh: {:,} EUR"
membership_satisfaction = "Spokojenost členských států: {:.2f} %"

intro = """
Ahoj {},
gratuluji - právě si se {} šéfem Evropské komise. Tvá síla a možnosti jsou prakticky neomezené.
Vládneš {} členským zemím a tvým úkolem je udržet je co nejdéle v EU.
"""

gender_dict = {
    "stal": {
        "m": "stal",
        "f": "stala",
        "n": "stalo"
    }
}

options = "Možnosti:"
higher_power_event = "Mimořádná událost"
higher_power_events = "Mimořádné události: {}"



import unittest
from eu2020.common.name import Name

names = {
    "Adam": "Adame",
    "Eliška": "Eliško",
    "Jana": "Jano",
    "Jarda": "Jardo",
    "Petr": "Petře",
    "Petra": "Petro",
    "Tomáš": "Tomáši",
}


class TestName(unittest.TestCase):

    def testName5p(self):
        for key in names.keys():
            name = Name(key, "m")
            self.assertEqual(names[key], name.get_name5p())


if __name__ == "__main__":
    unittest.main()



import eu2020.data.data as data


class Period:

    def __init__(self, year, month):
        self.year = year
        self.month = month


    def next(self):
        self.month = self.month + 1
        if self.month == len(data.months):
            self.month = 0
            self.year = self.year + 1


    def get_year(self):
        return self.year


    def get_month(self):
        return data.months[self.month]



import os
import eu2020.data.texts as texts


def print_text_in_box(text, ch="*"):
    boxw = len(text) + 4
    for _ in range(boxw):
        print(ch, end = "")
    print("\n{} {} {}".format(ch, text, ch))
    for _ in range(boxw):
        print(ch, end = "")
    print("\n")


def input_with_options(question, options):
    a = None
    while a not in options.keys():
        print(question)
        for o in options.keys():
            print(f"{o} - {options[o]}")
        a = input("> ")
    clear()
    return a


def proceed():
    input(texts.proceed)
    clear()


def clear():
    os.system('cls' if os.name == 'nt' else 'clear')



import re
import eu2020.data.texts as texts

ch5p = [
        (r"(.*)da$", "do"),
        (r"(.*)a$", "o"),
        (r"(.*)m$", "me"),
        (r"(.*)r$", "ře"),
        (r"(.*)š$", "ši"),
       ]


class Name:

    def __init__(self, name, gender):
        self.name = name
        self.gender = gender

    def get_name1p(self):
        return self.name

    def get_name5p(self):
        if self.gender != "n":
            for ch in ch5p:
                pattern = re.compile(ch[0])
                if pattern.match(self.name) is not None:
                    return pattern.sub(r"\1" + ch[1], self.name)
        return self.name

    def translate(self, word):
        t = texts.gender_dict[word]
        if (t is not None):
            return t[self.gender]
        return word



import random


class HigherPowerEvents: 

    events = []
    active_events = []


    def __init__(self, events):
        self.events = events

    
    def get_event(self):
        result = None
        i = random.randint(0, 2)
        if i == 0:
            i = random.randint(0, len(self.events) - 1)
            ev = self.events[i]
            if ev in self.active_events:
                self.active_events.remove(ev)
                result = ev["end"]
            else:
                self.active_events.append(ev)
                result = ev["start"]
        return result


    def get_current_events(self):
        result = ""
        for ev in self.active_events:
            result += ev["name"] + ", "
        return result



import unittest
from eu2020.common.period import Period


class TestPeriod(unittest.TestCase):

    def testFilter(self):
        period = Period(2020, 0)
        self.assertEqual(2020, period.get_year())
        self.assertEqual("Leden", period.get_month())
        period.next()
        self.assertEqual(2020, period.get_year())
        self.assertEqual("Únor", period.get_month())
        period.next()
        self.assertEqual(2020, period.get_year())
        self.assertEqual("Březen", period.get_month())
        period.next()
        self.assertEqual(2020, period.get_year())
        self.assertEqual("Duben", period.get_month())
        period.next()
        self.assertEqual(2020, period.get_year())
        self.assertEqual("Květen", period.get_month())
        period.next()
        self.assertEqual(2020, period.get_year())
        self.assertEqual("Červen", period.get_month())
        period.next()
        self.assertEqual(2020, period.get_year())
        self.assertEqual("Červenec", period.get_month())
        period.next()
        self.assertEqual(2020, period.get_year())
        self.assertEqual("Srpen", period.get_month())
        period.next()
        self.assertEqual(2020, period.get_year())
        self.assertEqual("Září", period.get_month())
        period.next()
        self.assertEqual(2020, period.get_year())
        self.assertEqual("Říjen", period.get_month())
        period.next()
        self.assertEqual(2020, period.get_year())
        self.assertEqual("Listopad", period.get_month())
        period.next()
        self.assertEqual(2020, period.get_year())
        self.assertEqual("Prosinec", period.get_month())
        period.next()
        self.assertEqual(2021, period.get_year())
        self.assertEqual("Leden", period.get_month())
        period.next()


if __name__ == "__main__":
    unittest.main()



class Parties:

    parties = {}


    def __init__(self, parties):
        self.parties = parties


    def get_satisfaction_pct(self):
        i = 0
        s = 0
        for c in self.parties.keys():
            s += self.parties[c]["satisfaction_pct"]
            i += 1
        return s / i


    def update_satisfaction(self, satisfaction):
        for c in satisfaction.keys():
            self.parties[c]["satisfaction_pct"] += satisfaction[c]
            if self.parties[c]["satisfaction_pct"] < 0:
                self.parties[c]["satisfaction_pct"] = 0


    def get_budget_contribution(self):
        result = 0
        for c in self.parties.keys():
            result += self.parties[c]["budget_contribution"]
        return result


    def get_budget_consumption(self):
        result = 0
        for c in self.parties.keys():
            result += self.parties[c]["budget_consumption"]
        return result



class Budget:

    parties = []
    extra_income = 0
    extra_outcome = 0
    dept = 0


    def add_parties(self, parties):
        self.parties.append(parties)


    def get_balance(self):
            return self.get_income() - self.get_outcome()


    def get_dept(self):
            return self.dept


    def get_income(self):
        result = 0
        for p in self.parties:
            result += p.get_budget_contribution()
        return result + self.extra_income


    def get_outcome(self):
        result = 0
        for p in self.parties:
            result += p.get_budget_consumption()
        return result + self.extra_outcome


    def add_extra_income(self, amt):
        self.extra_income += amt


    def add_extra_outcome(self, amt):
        self.extra_outcome += amt


    def update_balance(self):
        self.dept -= self.get_income() - self.get_outcome()
        self.extra_income = 0
        self.extra_outcome = 0
    


import random
import eu2020.data.texts as texts
import eu2020.common.utils as utils
from eu2020.common.parties import Parties


class EventProcessor:

    events = []
    all_parties = Parties({})


    def add_events(self, events):
        self.events.append(events)
        parties = events.get_parties().parties
        for p in parties.keys():
            if p not in self.all_parties.parties.keys():
                self.all_parties.parties[p] = parties[p]


    def process_events(self, budget):
        for evs in self.events:
            for _ in range(random.randint(1, 3)):
                ev = evs.get_event()
                if ev is not None:
                    country = evs.get_parties().parties[ev["party"]]
                    utils.print_text_in_box(country["name"])
                    self.make_decision(ev, self.all_parties, budget)


    def make_decision(self, ev, members, budget):
        print(ev["description"])
        print()
        dopt = {}
        for opt in ev["options"]:
            dopt[opt["key"]] = opt["description"]
        g = utils.input_with_options(texts.options, dopt)
        options = list(filter(lambda o: o["key"] == g, ev["options"]))
        
        # Satisfaction
        members.update_satisfaction(options[0]["impact"]["satisfaction"])

        # Budget
        b = options[0]["impact"]["budget"]
        budget.add_extra_income(b)


    def next_period(self):
        for evs in self.events:
            evs.next_period()



import eu2020.data.texts as texts
import eu2020.data.data as data
import eu2020.data.events as events
import eu2020.common.utils as utils
from eu2020.common.name import Name
from eu2020.common.period import Period
from eu2020.common.higher_power_events import HigherPowerEvents
from eu2020.common.parties_events import PartiesEvents
from eu2020.common.parties import Parties
from eu2020.common.budget import Budget
from eu2020.common.event_processor import EventProcessor


def main():
    members = Parties(data.member_countries)
    administration = Parties(data.eu_administration)
    deep_state = Parties(data.deep_state)

    hp_events = HigherPowerEvents(events.higher_power_events)
    m_events = PartiesEvents(events.member_country_events, members, hp_events)
    deep_events = PartiesEvents(events.deep_state_events, deep_state, hp_events)

    event_processor = EventProcessor()
    event_processor.add_events(m_events)
    event_processor.add_events(deep_events)

    budget = Budget()
    budget.add_parties(members)
    budget.add_parties(administration)
    budget.add_parties(deep_state)

    data.period = Period(2020, 0)

    utils.clear()
    print_app_name()

    setup()
    print(texts.intro.format(data.name.get_name5p(), data.name.translate("stal"), len(data.member_countries)))
    utils.proceed()

    while True:
        utils.print_text_in_box(texts.period.format(data.period.get_month(), data.period.get_year()))
        print_budget(budget)
        print_membership_satisfaction(members)
        print_hp_events(hp_events)
        print()
        utils.proceed()

        event_processor.process_events(budget)

        hpev = hp_events.get_event()
        if hpev != None:
            utils.print_text_in_box(texts.higher_power_event, "!")
            print(hpev + "\n")
            utils.proceed()

        data.period.next()
        event_processor.next_period()
        if data.period.month == 0:
            budget.update_balance()


def print_budget(budget):
    print(texts.budget.format(data.period.get_year(), budget.get_income(), budget.get_balance(), budget.get_dept()))


def print_membership_satisfaction(members):
    print(texts.membership_satisfaction.format(members.get_satisfaction_pct()))


def print_hp_events(hp_events):
    ev = hp_events.get_current_events()
    if ev != "":
        print(texts.higher_power_events.format(hp_events.get_current_events()))


def setup():
    n = input(f"{texts.what_is_your_name}\n> ")
    print()
    g = utils.input_with_options(texts.what_is_your_gender, data.gender)
    data.name = Name(n, g)


def print_app_name():
    utils.print_text_in_box(texts.app_name)



gender = {"m": "muž", "f": "žena", "n": "nebinární"}

months = ["Leden", "Únor", "Březen", "Duben", "Květen", "Červen", "Červenec", "Srpen", "Září", "Říjen", "Listopad", "Prosinec"]

name = None
period = None

member_countries = {
    "BE": {
        "name": "Belgie",
        "satisfaction_pct": 67,
        "budget_contribution": 5_664_000_000,
        "budget_consumption": 3_664_000_000,
    },
    "BG": {
        "name": "Bulharsko",
        "satisfaction_pct": 77,
        "budget_contribution": 560_000_000,
        "budget_consumption": 1_000_000_000,
    },
    "CZ": {
        "name": "Česko",
        "satisfaction_pct": 52,
        "budget_contribution": 2_224_000_000,
        "budget_consumption": 3_000_000_000,
    },
    "DK": {
        "name": "Dánsko",
        "satisfaction_pct": 67,
        "budget_contribution": 2_224_000_000,
        "budget_consumption": 1_224_000_000,
    },
    "EE": {
        "name": "Estonsko",
        "satisfaction_pct": 67,
        "budget_contribution": 224_000_000,
        "budget_consumption": 500_000_000,
    },
    "FI": {
        "name": "Finsko",
        "satisfaction_pct": 67,
        "budget_contribution": 2_896_000_000,
        "budget_consumption": 2_000_000_000,
    },
    "FR": {
        "name": "Francie",
        "satisfaction_pct": 51,
        "budget_contribution": 32_288_000_000,
        "budget_consumption": 20_000_000_000,
    },
    "HR": {
        "name": "Chorvatsko",
        "satisfaction_pct": 67,
        "budget_contribution": 96_000_000,
        "budget_consumption": 150_000_000,
    },
    "IE": {
        "name": "Irsko",
        "satisfaction_pct": 67,
        "budget_contribution": 2_144_000_000,
        "budget_consumption": 1_500_000_000,
    },
    "IT": {
        "name": "Itálie",
        "satisfaction_pct": 58,
        "budget_contribution": 24_576_000_000,
        "budget_consumption": 20_000_000_000,
    },
    "CY": {
        "name": "Kypr",
        "satisfaction_pct": 67,
        "budget_contribution": 272_000_000,
        "budget_consumption": 500_000_000,
    },
    "LT": {
        "name": "Litva",
        "satisfaction_pct": 83,
        "budget_contribution": 432_000_000,
        "budget_consumption": 800_000_000,
    },
    "ES": {
        "name": "Španělsko",
        "satisfaction_pct": 66,
        "budget_contribution": 16_288_000_000,
        "budget_consumption": 12_000_000_000,
    },
    "SE": {
        "name": "Švédsko",
        "satisfaction_pct": 72,
        "budget_contribution": 4_528_000_000,
        "budget_consumption": 3_528_000_000,
    },
    "HU": {
        "name": "Maďarsko",
        "satisfaction_pct": 67,
        "budget_contribution": 1_568_000_000,
        "budget_consumption": 2_568_000_000,
    },
    "MT": {
        "name": "Malta",
        "satisfaction_pct": 67,
        "budget_contribution": 96_000_000,
        "budget_consumption": 70_000_000,
    },
    "DE": {
        "name": "Německo",
        "satisfaction_pct": 69,
        "budget_contribution": 35_856_000_000,
        "budget_consumption": 23_856_000_000,
    },
    "NL": {
        "name": "Nizozemsko",
        "satisfaction_pct": 66,
        "budget_contribution": 7_216_000_000,
        "budget_consumption": 6_000_000_000,
    },
    "PL": {
        "name": "Polsko",
        "satisfaction_pct": 84,
        "budget_contribution": 5_920_000_000,
        "budget_consumption": 7_920_000_000,
    },
    "PT": {
        "name": "Portugalsko",
        "satisfaction_pct": 67,
        "budget_contribution": 2_624_000_000,
        "budget_consumption": 2_000_000_000,
    },
    "AT": {
        "name": "Rakousko",
        "satisfaction_pct": 67,
        "budget_contribution": 4_240_000_000,
        "budget_consumption": 3_240_000_000,
    },
    "RO": {
        "name": "Rumunsko",
        "satisfaction_pct": 67,
        "budget_contribution": 1_984_000_000,
        "budget_consumption": 2_984_000_000,
    },
    "GR": {
        "name": "Řecko",
        "satisfaction_pct": 53,
        "budget_contribution": 3_696_000_000,
        "budget_consumption": 3_000_000_000,
    },
    "SK": {
        "name": "Slovensko",
        "satisfaction_pct": 70,
        "budget_contribution": 1_072_000_000,
        "budget_consumption": 1_800_000_000,
    },
    "SI": {
        "name": "Slovinsko",
        "satisfaction_pct": 67,
        "budget_contribution": 576_000_000,
        "budget_consumption": 1_000_000_000,
    },
    "LV": {
        "name": "Lotyšsko",
        "satisfaction_pct": 67,
        "budget_contribution": 272_000_000,
        "budget_consumption": 500_000_000,
    },
    "LU": {
        "name": "Lucembursko",
        "satisfaction_pct": 67,
        "budget_contribution": 464_000_000,
        "budget_consumption": 380_000_000,
    },
}

eu_administration = {
    "AD": {
        "name": "Administrative",
        "satisfaction_pct": 50,
        "budget_contribution": 0,
        "budget_consumption": 10_000_000_000,
    },
    "PR": {
        "name": "EU Propagation",
        "satisfaction_pct": 50,
        "budget_contribution": 0,
        "budget_consumption": 10_000_000_000,
    },
    "ED": {
        "name": "European Defence Agency",
        "satisfaction_pct": 50,
        "budget_contribution": 0,
        "budget_consumption": 5_000_000_000,
    },
    "SP": {
        "name": "Special projects",
        "satisfaction_pct": 50,
        "budget_contribution": 0,
        "budget_consumption": 3_000_000_000,
    },
}

deep_state = {
    "GDA": {
        "name": "Globalisti z Davosu",
        "satisfaction_pct": 50,
        "budget_contribution": 0,
        "budget_consumption": 0,
    },
    "SOR": {
        "name": "George Soros",
        "satisfaction_pct": 50,
        "budget_contribution": 0,
        "budget_consumption": 0,
    },
    "NBM": {
        "name": "Nadace Billa a Melindy Gatesových",
        "satisfaction_pct": 50,
        "budget_contribution": 0,
        "budget_consumption": 0,
    },
    "NEZ": {
        "name": "Neziskové organizace",
        "satisfaction_pct": 50,
        "budget_contribution": 0,
        "budget_consumption": 4_816_000_000,
    },
    "WHO": {
        "name": "Světová Zdravotnická Organizace",
        "satisfaction_pct": 50,
        "budget_contribution": 0,
        "budget_consumption": 2_000_000_000,
    },
    "BIT": {
        "name": "Big Tech Korporace",
        "satisfaction_pct": 50,
        "budget_contribution": 0,
        "budget_consumption": 0,
    },
    "BIP": {
        "name": "Big Pharma Korporace",
        "satisfaction_pct": 50,
        "budget_contribution": 0,
        "budget_consumption": 0,
    },
}



from setuptools import setup, find_namespace_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

with open("VERSION", "r") as fh:
  version = fh.read()

with open("requirements.txt", "r") as fh:
    requirements = fh.readlines()

setup(
    name='eu2020',
    version=version,
    author='Jaroslav Beran',
    author_email='jaroslav.beran@gmail.com',
    description='Textová hra Evropská Unie 2020',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/berk76/eu2020',
    project_urls={
        "Bug Tracker": "https://github.com/berk76/eu2020/issues"
    },
    entry_points={
        'console_scripts': [
            'eu2020 = eu2020.app:main',
        ],
    },
    license='GPL-3.0',
    packages=find_namespace_packages(include=['eu2020', 'eu2020.*']),
    install_requires=requirements,
)



higher_power_events = [
    {
        "key": "PAN",
        "name": "Pandemie",
        "start": "Propukla pandemie nové neznámé choroby.",
        "end": "Zdá se, že pandemie ustala.",
    },
    {
        "key": "POV",
        "name": "Povodně",
        "start": "Po celé Evropě propukly povodně.",
        "end": "Povodně se podařilo zvládnout.",
    },
    {
        "key": "INF",
        "name": "Inflace",
        "start": "Evropa se potýká s vysokou mírou inflace.",
        "end": "Inflaci se podařilo zvládnout.",
    },
    {
        "key": "HRE",
        "name": "Hospodářská recese",
        "start": "Evropou zmítá hospodářská recese, podniky propouštějí.",
        "end": "Hospodářskou recesi se podařilo zvládnout.",
    },
]


member_country_events = [
    {
        "party": "CZ",
        "description": "Některé strany tlačí na deklaraci proti střetu zájmů českého premiéra.",
        "options": [
            {
                "key": "p",
                "description": "přijmout deklaraci",
                "impact": {
                    "satisfaction": {"CZ": -5, "HU": -3, "PL": -3, "SK": -1},
                    "budget": 0,
                }
            },
            {
                "key": "o",
                "description": "odmítnout deklaraci",
                "impact": {
                    "satisfaction": {"CZ": 5, "HU": 1, "PL": 1, "SK": 1},
                    "budget": 0,
                }
            },
            {
                "key": "n",
                "description": "nerozhodnout teď",
                "impact": {
                    "satisfaction": {},
                    "budget": 0,
                }
            },
        ]
    },
    {
        "party": "HU",
        "description": "Maďarsko potlačuje práva LBQT menšin. LBQT neziskovka navrhuje odsoudit Maďarsko za porušování práv menšin",
        "options": [
            {
                "key": "p",
                "description": "přijmout deklaraci",
                "impact": {
                    "satisfaction": {"CZ": -3, "HU": -8, "PL": -3, "SK": -1},
                    "budget": 0,
                }
            },
            {
                "key": "k",
                "description": "udělit pokutu 200 000 000 EUR",
                "impact": {
                    "satisfaction": {"CZ": -3, "HU": -8, "PL": -3, "SK": -1},
                    "budget": 200_000_000,
                }
            },
            {
                "key": "o",
                "description": "odmítnout deklaraci",
                "impact": {
                    "satisfaction": {"CZ": 3, "HU": 8, "PL": 3, "SK": 1},
                    "budget": 0,
                }
            },
            {
                "key": "n",
                "description": "nerozhodnout teď",
                "impact": {
                    "satisfaction": {},
                    "budget": 0,
                }
            },
        ]
    },
]


deep_state_events = [
    {
        "party": "SOR",
        "description": "George Soros požaduje rychlejší přísun migrantů z Afriky. Nabízí příspěvek 1 000 000 000 EUR jako podporu pro lidskoprávní organizace.",
        "options": [
            {
                "key": "p",
                "description": "přijmout",
                "impact": {
                    "satisfaction": {"CZ": -3, "HU": -3, "PL": -3, "SK": -3, "AT": -3, "IT": -3, "FR": -2},
                    "budget": 1_000_000_000,
                }
            },
            {
                "key": "o",
                "description": "odmítnout",
                "impact": {
                    "satisfaction": {"SOR": -10, "NEZ": -5, "WHO": -2},
                    "budget": 0,
                }
            },
            {
                "key": "n",
                "description": "nerozhodnout teď",
                "impact": {
                    "satisfaction": {"SOR": -3,},
                    "budget": 0,
                }
            },
        ]
    },
    {
        "party": "WHO",
        "description": "V souvislosti s probíhající pandemií WHO založila nadační fond COVAX, který má financovat nákup a distiribuci vakcín po celém světe. WHO žádá Evropskou unii o příspěvek 10 000 000 000 EUR",
        "higher_power_cond": "PAN",
        "options": [
            {
                "key": "p",
                "description": "přijmout",
                "impact": {
                    "satisfaction": {"WHO": 20, "NEZ": 10, "NBM": 10, "SOR": 5, "GDA": 5, "BIP": 10},
                    "budget": -10_000_000_000,
                }
            },
            {
                "key": "o",
                "description": "odmítnout",
                "impact": {
                    "satisfaction": {"WHO": -10, "NEZ": -5, "NBM": -5, "SOR": -5, "GDA": -5, "BIP": -5},
                    "budget": 0,
                }
            },
            {
                "key": "n",
                "description": "nerozhodnout teď",
                "impact": {
                    "satisfaction": {"WHO": -10},
                    "budget": 0,
                }
            },
        ]
    },
]



import random

class PartiesEvents:

    events = []
    parties = None
    hp_events = None


    def __init__(self, events, parties, hp_events):
        self.events = events
        for ev in self.events:
            ev["wait"] = 0
        self.parties = parties
        self.hp_events = hp_events


    def event_filter(self, ev):
        if "higher_power_cond" in ev:
            met = False
            for he in self.hp_events.active_events:
                if ev["higher_power_cond"] == he["key"]:
                    met = True
                    break
            if met == False:
                return False

        return ev["wait"] == 0


    def get_event(self):
        filtered = list(filter(self.event_filter, self.events))
        if (len(filtered) > 0):
            n = random.randint(0, len(filtered) - 1)
            ev = filtered[n]
            ev["wait"] = 1
            return ev
        else:
            return None


    def get_parties(self):
        return self.parties


    def next_period(self):
        for ev in self.events:
            if ev["wait"] > 0:
                ev["wait"] -= 1
